//-----------------------------------------------------------------
// Archivo de cdigo C++ - AlladronAnimado.cpp
//-----------------------------------------------------------------
//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "AlladronAnimado.h"
#include <math.h>

//-----------------------------------------------------------------
// Funciones del NucleoMotor
//-----------------------------------------------------------------
BOOL FJ_InicializarJuego(HINSTANCE hInstancia)
{
  // Crea el ncleo del juego
  g_pJuego = new NucleoMotor(hInstancia,
    TEXT("Al ladrn!"), TEXT("Al ladrn!"),
    IDI_ALLADRON, IDI_ALLADRON_SM,
    640, 480, 10); // frecuencia 10 fps equivale a 1 ciclo cada 100 milisegundos

  // si no se pudo crear el objeto juego
  // no se inicia
  if (g_pJuego == NULL)
    return false;

  // Se inicializa el joystick
  if (!g_pJuego->InicializarJoystick())
    return false;

  return true;
}
void FJ_ComenzarJuego(HWND hWindow)
{

  // Obtiene un handle al contexto de dispositivo
  HDC hDC = GetDC(hWindow);

  // Crea una serie de mapa de bits
  // Los mapas de bits son de profundidad 24 bits

  // El objeto impacto se mostrar cada vez que
  // se produzca una colisin
  g_pImpactoTomate = new cBMP(hDC, TEXT("Recursos//impactoTomate.bmp"));

  // Fondo del juego
  g_pFondoJuego = new cBMP(hDC, IDB_BMP1, g_pJuego->ObtenerInstancia());

  // Los tres tipos de ladrones
  // uno para el que va hacia la izquierda (0)
  // otro el que va hacia la derecha (1)
  // otro con las manos levantadas (2)

  g_pLadron[0] = new cBMP(hDC, TEXT("Recursos//ladronD.bmp"));
  g_pLadron[1] = new cBMP(hDC, TEXT("Recursos//ladronI.bmp"));
  g_pLadron[2] = new cBMP(hDC, TEXT("Recursos//ladronDetenido.bmp"));

  // Bitmap del tomate que se arroja
  g_pTomate = new cBMP(hDC, TEXT("Recursos//tomate.bmp"));

  // Bitmap del rbol
  g_pArbol = new cBMP(hDC, TEXT("Recursos//arbol.bmp"));

  // Bitmap del auto
  g_pAuto = new cBMP(hDC, TEXT("Recursos//auto.bmp"));

  // Bitmap del auto robado
  g_pAutoRobado = new cBMP(hDC, TEXT("Recursos//autoRobado.bmp"));

  // Bitmap de la mirilla para disparar (es un tomate mvil)
  g_pMirilla = new cBMP(hDC, TEXT("Recursos//mirilla.bmp"));

  // Contadores para cada jugador
  g_iAutosRobados = 0;
  g_iLadronesDetenidos = 0;

  // Inicializar los contadores del juego
  FJ_NuevoJuego();

  // Bfer auxiliar de repintado
  // Crea un contexto de dispositivo y un
  // mapa de bits adicional
  g_hBuferAuxDC = CreateCompatibleDC(hDC);
  g_hBuferAuxBitmap = CreateCompatibleBitmap(hDC,
      g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura());
  SelectObject(g_hBuferAuxDC , g_hBuferAuxBitmap);

 }
void FJ_FinalizarJuego()
{
  // Elimina los elementos del bfer auxiliar
  DeleteObject(g_hBuferAuxBitmap);
  DeleteObject(g_hBuferAuxDC);

  // Elimina objetos bmp
  for (int i = 0; i < 3;i++)
      delete g_pLadron[i];

  delete g_pTomate;
  delete g_pAuto;
  delete g_pAutoRobado;
  delete g_pMirilla;
  delete g_pImpactoTomate;

  // Limpieza del vector de sprites
  g_pJuego->LimpiarVector();

  // elimina el mapa de bits de fondo
  delete g_pFondoJuego;

  // Elimina el objeto
  delete g_pJuego;

  // Cierra el dispositivo
  g_pJuego->MIDI(CERRARMIDI);
}
void FJ_ReactivarJuego(HWND hWindow)
{
    // Capturar el joystick
    g_pJuego->CapturarJoystick();

    // reactiva la msica de fondo
    // con la msica que se estaba
    // reproduciendo en el momento
    // de la pausa
    g_pJuego->MIDI(PLAYMIDI,NULL,false);
}
void FJ_DetenerJuego(HWND hWindow)
{
    // Liberar el joystick
     g_pJuego->LiberarJoystick();

    // Suspende la msica de fondo
    g_pJuego->MIDI(PAUSARMIDI);
}

void FJ_CicloJuego()
{
    // este cdigo se ejecuta cada vez que el
    // ncleo motor detecta que corresponde
    // ejecutar un nuevo ciclo (en base a la
    // frecuencia que hemos informado al crear
    // el objeto juego)

    // si no finaliz el juego se realiza un ciclo
    // esto implica:
    // 1) la aplicacin de los algoritmos
    // que comandan el movimiento del ladrn
    // 2) la actualizacin de los sprites
    // 3) el dibujo de la pantalla
    if (!g_bFinJuego)
    {
        // De modo aleatorio el ladrn se traslada
        // de rbol en rbol hasta llegar al
        // auto

        // si el ladrn est quieto (velocidad 0,0)
        // puede entrar en movimiento de modo aleatorio
        POINT ptVelActual = g_pcsLadron->ObtenerVelocidad();
        if (ptVelActual.x == 0 && ptVelActual.y == 0 && g_iPosActual != 12)
        {
            // El ladrn no se mueve en todos los
            // ciclos, ms bien al revs se queda
            // ms tiempo oculto que en movimiento
            // dado que la funcin rand() utiliza % 5
            // esto implica que en promedio el objeto
            // se mueve 1 de cada 5 ciclos
            // para moverse as de modo inesperado
            // comprobamos si le toca moverse
            if (rand() % 5 == 0)
            {
            // Elige el nuevo destino para el ladrn
            // siempre es un valor en ascenso desde
            // el punto de origen actual del ladrn
            // los rboles tienen el destino de 0 a 11
            // y el auto es el destino 12
            // Cuntos destinos avanza?

                int iDestino =  rand() % 3 + 1 + g_iPosActual;
                // el destino debe ser 12 como mximo
                // de 0 a 11 son los rboles y el
                // ndice 12 es el auto (destino final)
                // Como mximo vale el valor 12
                iDestino = min(iDestino,12);

                // para que el ladrn vaya al destino
                // elegido se recalcula su rea de accin
                // utilizando el rectngulo del punto de
                // origen y el punto de destino
                RECT rcAreaAccion;
                UnionRect(&rcAreaAccion, &g_rDestinos[g_iPosActual],
                            &g_rDestinos[iDestino]);

                // Define el rectngulo de accin del sprite
                g_pcsLadron->AsignarAreaAccion(rcAreaAccion);

                // La velocidad est formada por dos componentes
                // x e y. De esta manera no slo se define velocidad
                // sino tambin la direccin
                // Asignarle una velocidad proporcional
                // para que se dirija al destino
                // la velocidad sirve para dos acciones: dirigir el
                // objeto (valor x e y)  y para darle movimiento
                RECT rcActual = g_pcsLadron->ObtenerPosicion();
                int iDifx =  g_rDestinos[iDestino].left - rcActual.left;
                int iDify =  g_rDestinos[iDestino].top - rcActual.top;

                // Se reduce un poco la velocidad para que no sea
                // demasiado rpida y no se de la posibilidad
                // de detenerlo con un proyectil
                g_pcsLadron->AsignarVelocidad(iDifx/10,iDify/10);
                // veamos qu imagen se muestra

                // si la velocidad en el eje x es negativa
                // quiere decir que el ladrn corre de
                // derecha a izquierda, en caso contrario lo
                // har de izquierda a derecha
                // esto nos permite elegir la imagen correcta
                if (iDifx < 0)
                    g_pcsLadron->CambiarImagen(g_pLadron[1]);
                else
                    g_pcsLadron->CambiarImagen(g_pLadron[0]);

                // Se guarda el nuevo destino del ladrn
                // para el siguiente movimiento
                g_iPosActual = iDestino;

            }

        }

        // Actualiza las posiciones de los todos
        // los sprites en base a sus posiciones y velocidades
        g_pJuego->ActualizarSprites();
        // contamos los ciclos ejecutados
        // (siempre es un valor que puede servir
        // en las pruebas del cdigo)
        ++g_iContBucle;

        // Pintar en el contexto de dispositivo
        // auxiliar para eliminar
        // la sensacin de parpadeo
        FJ_PintarJuego(g_hBuferAuxDC);

        // Pintura real de la pantalla en una nica operacin
        HWND hVentana = g_pJuego->ObtenerVentana();
        HDC hDC = GetDC(hVentana);

        // se utilizan coordenadas 0, 0
        // porque se realiza copia completa
        // de un rea a otra
        BitBlt(hDC,
            0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura(),
            g_hBuferAuxDC, 0, 0,
            SRCCOPY);

        ReleaseDC(hVentana, hDC);

        // Se depuran los sprites marcados para eliminar
        g_pJuego->DepurarSprites();
    }
}
void FJ_PintarJuego(HDC hDC)
{

    POINT ptOrigen = {0, 0};
    // El fondo del juego se pinta primero
    g_pFondoJuego->Dibujar(hDC, ptOrigen);

    // Ahora dibuja los sprites desde el vector
    // que pertenece al NucleoMotor
    // el orden de pintado es el orden Z
    g_pJuego->DibujarSprites(hDC);

    // se informa el record del juego
    TCHAR szTexto[90];
    POINT ptx = g_pcsLadron->ObtenerVelocidad();
    wsprintf(szTexto, "Autos robados: %d - Ladrones detenidos %d - F3 Reinicio, F4 Contadores a 0",
       g_iAutosRobados, g_iLadronesDetenidos);
    TextOut(hDC, 0, 0, szTexto, strlen(szTexto));

}
void FJ_LButtonDown(LPARAM lParam)
{
}
void FJ_RButtonUp(LPARAM lParam)
{
}
void FJ_LButtonUp(LPARAM lParam)
{
}
void FJ_MouseMove(LPARAM lParam)
{
    // se sigue el movimiento del cursor del ratn
    // la mirilla es un tomate
    g_pcsMirilla->AsignarPosicion(LOWORD(lParam) - (g_pMirilla->ObtenerAncho() / 2),
                                  HIWORD(lParam) - (g_pMirilla->ObtenerAltura() / 2));

}
void FJ_RButtonDown(LPARAM lParam)
{
    // quedan proyectiles disponibles?
    if (g_iTomatesArrojados == g_iQShots)
        return;

    cSprite * pcSprite;

    // Hay algn sprite en donde est el cursor?
    pcSprite = g_pJuego->HitTest(LOWORD(lParam), HIWORD(lParam));

    // el tomatazo es donde est el cursor del ratn
    g_pcsTomate[g_iTomatesArrojados]->AsignarPosicion(LOWORD(lParam) - g_pImpactoTomate->ObtenerAncho() / 2,
                    HIWORD(lParam) - g_pImpactoTomate->ObtenerAltura() / 2);

    // el tomate queda aplastado al impactar contra algo
    // cambiamos la imagen del tomate para que
    // parezca aplastado
    g_pcsTomate[g_iTomatesArrojados]->CambiarImagen();

    // controlamos los tomates arrojados
    g_iTomatesArrojados++;
    PlaySound((LPCSTR) IDW_DISPARO, g_pJuego->ObtenerInstancia(),
                            SND_ASYNC | SND_RESOURCE);
    // Si no hay un sprite, no interesa
    // quiere decir que se ha fallado
    if ( pcSprite != NULL)
    {
        // el sprite impactado que nos
        // interesado es el LADRON
        if (pcSprite->ObtenerID() == LADRON)
        {
            // Se acert con un ladrn
            g_iLadronesDetenidos++;
            PlaySound((LPCSTR) IDW_OK, g_pJuego->ObtenerInstancia(),
                            SND_ASYNC | SND_RESOURCE);
            // manos arriba
            g_pcsLadron->CambiarImagen(g_pLadron[2]);
            g_pcsLadron->AsignarCantidadImagenes(1);
            // que se quede quieto
            // le cambiamos la velocidad
            g_pcsLadron->AsignarVelocidad(0,0);
            // ya no puede recibir ms impactos
            g_pcsLadron->AsignarDensidad(0);

            g_iPosActual = 12;

        }
        // si el ladrn est dentro del auto y hemos
        // impactado con el AUTO, tambin
        // hemos detenido al ladrn
        // si la posicin actual del ladrn es 12
        // quiere decir que est dentro del auto
        if  ((pcSprite->ObtenerID() == AUTO) && (g_iPosActual == 12))
        {
            // Se acert con un ladrn
            // cuando estaba subido al auto
            g_iLadronesDetenidos++;
            PlaySound((LPCSTR) IDW_OK, g_pJuego->ObtenerInstancia(),
                            SND_ASYNC | SND_RESOURCE);
            // manos arriba
            g_pcsLadron->CambiarImagen(g_pLadron[2]);
            g_pcsLadron->AsignarCantidadImagenes(1);
            // que se quede quieto
            g_pcsLadron->AsignarVelocidad(0,0);
            // volver a visualizar toda la figura
            // del ratn
            g_pcsLadron->AsignarVisibilidad(true);
            // ya no puede recibir ms impactos
            g_pcsLadron->AsignarDensidad(0);

            // Qu hacemos con el auto?
            // Coloca nuevamente el auto en el puesto inicial
            pcSprite->CambiarImagen();
            pcSprite->AsignarPosicion(pcSprite->ObtenerPosicionInicial());
            // lo dejamos quieto en la posicin inicial
            pcSprite->AsignarVelocidad(0,0);
        }

    }
}
void FJ_CheckTeclado()
{
    // F3: nuevo juego del mismo jugador
    if (GetAsyncKeyState(VK_F3) < 0)
        FJ_NuevoJuego();
    // F4: limpia contadores, para
    // que juegue otro
    if (GetAsyncKeyState(VK_F4) < 0)
    {
        g_iAutosRobados = 0;
        g_iLadronesDetenidos = 0;
        FJ_NuevoJuego();
    }


}
void FJ_TeclaCaracter(WPARAM wParam)
{
}

void FJ_CheckJoystick(WORD wJoyEstado)
{
}

APC_ACCION FJ_NotificarColision(TC_COLISION tcColision, cSprite* pSprite, cSprite* pSpriteImpactado)
{
  // Qu colisiones nos interesan?
  // 1) auto - ladrn: esto implica que el ladrn lleg al auto
  // 2) auto con lmite del juego: esto implica que el jugador perdi

  // Si hay colisin de auto-ladrn
  // es porque el ladrn ha llegado al auto
  // y salvo un impacto en el rea sensible dentro
  // del auto, el juego terminar al desaparecer el auto

     switch (tcColision)
     {
        case TC_SPRITE:
            // Colisin ladron-auto = robo
            if ((pSprite->ObtenerID() == LADRON && pSpriteImpactado->ObtenerID() == AUTO) ||
               (pSprite->ObtenerID() == AUTO && pSpriteImpactado->ObtenerID() == LADRON) )
            {
                // el ladron se sube al auto
                g_pcsAuto->CambiarImagen();
                // hacemos que la imagen del ladrn
                // sea no visible, ya que est dentro
                // del auto
                g_pcsLadron->AsignarVisibilidad(false);
                // El coche acelera y se marcha
                // hacia la izquierda
                g_pcsAuto->AsignarVelocidad(-15,-1);
            }
            break;
        case TC_LIMITES:
            if (pSprite->ObtenerID() == AUTO)
            {
                // el auto llega al final de la pantalla del juego
                // esto indica que el jugador ya pwerdi
                g_iAutosRobados++;
                PlaySound((LPCSTR) IDW_KO, g_pJuego->ObtenerInstancia(),
                            SND_ASYNC | SND_RESOURCE);
                g_bFinJuego = true;
            }
            break;
        // otras opciones
        // reservamos para uso futuro
    }
    return APC_NOACCION;
}


void FJ_NuevoJuego()
{
  // Funcin de limpieza de las variables
  // para reiniciar un nuevo
  g_pJuego->LimpiarVector();    // Se limpian los sprites del vector
  g_iTomatesArrojados = 0;      // Tomates arrojados
  g_bFinJuego = false;          // Indicador de fin de juego

  // el rea de movimiento de los sprites es toda
  // la pantalla
  RECT rcAreaJuego = {0, 0,
        g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura()};

  // Definimos un hot spot para
  // los distintos sprites
  RECT rcHotSpot;

  // Objeto rboles
  // Asignamos las posiciones de los rboles
  // en filas y columnas de rboles
  // Los rboles son destinos intermedios en los que
  // se `puede esconder el ladrn
  g_iDestinos = 0;
  for (int i = 0; i < 3 ; i++)
  {
    int iFila = i  * 125 + 10;
    // rboles por fila
    for (int j = 0; j < 4; j++)
    {
        g_pcsArbol[g_iDestinos] = new cSprite(g_pArbol, rcAreaJuego, AFL_DESAPARECER);
        g_pcsArbol[g_iDestinos]->AsignarOrdenZ(3);
        g_pcsArbol[g_iDestinos]->AsignarID(ARBOL);
        // El rbol, por supuesto, es esttico
        g_pcsArbol[g_iDestinos]->AsignarVelocidad(0,0);
        // El rbol tiene un rea sensible
        // menor que su tamao
        SetRect(&rcHotSpot, 20, 50, 40, 100);
        g_pcsArbol[g_iDestinos]->AsignarHotSpot(rcHotSpot);

        int iCol = j * 140;
        g_pcsArbol[g_iDestinos]->AsignarPosicion(iCol, iFila);

        // Guardamos las posiciones de los rboles
        g_rDestinos[g_iDestinos] = g_pcsArbol[g_iDestinos]->ObtenerPosicion();
        // se agrega el sprite al vector
        g_pJuego->AgregarSprite(g_pcsArbol[g_iDestinos]);

        g_iDestinos++;
    }
  }


  // Se aade el ladrn
  // Los ladrones corren hacia la izquierda y
  // hacia la derecha, esto lo resolvemos con dos bitmaps
  g_pcsLadron = new cSprite(g_pLadron[0],
                    rcAreaJuego, AFL_DETENER);

  // el orden Z es importante para definir
  // si pasa por adelante o por atrs de
  // los rboles (que tienen orden Z = 3
  g_pcsLadron->AsignarOrdenZ(2);
  // El ladrn tiene un rea sensible
  // menor que su tamao
  SetRect(&rcHotSpot, 20, 50, 40, 100);
  g_pcsLadron->AsignarHotSpot(rcHotSpot);

  // los ladrones tienen identificadores 100
  g_pcsLadron->AsignarID(LADRON);

  // Cada ladrn se oculta en un rbol inicial aleatorio
  // entre 0 y 2 (para que no est tan cerca del auto)
  int iArbol = rand() % 3;
  g_pcsLadron->AsignarPosicion(g_rDestinos[iArbol]);

  // El mapa de bits est formado por 4 imgenes
  g_pcsLadron->AsignarCantidadImagenes(4);

  // cada 2 ciclos se cambia el fotograma
  g_pcsLadron->AsignarDemoraAnimacion(2);

  // Para cada ladrn se mantiene el ndice del
  // rbol en el que se oculta
  g_iPosActual = iArbol;



// se agrega el sprite
  g_pJuego->AgregarSprite(g_pcsLadron);



  // Se coloca el auto
  // Objeto auto
  POINT ptPos = {g_pJuego->ObtenerAncho() - g_pAuto->ObtenerAncho() - 200,
                 g_pJuego->ObtenerAltura() - g_pAuto->ObtenerAltura() };
  POINT ptVelo = {0, 0};

  // hot spot del auto
  SetRect(&rcHotSpot, 20, 10, 180, 50);
  g_pcsAuto = new cSprite(g_pAuto, ptPos, rcHotSpot, ptVelo, 1,
                rcAreaJuego, AFL_DESAPARECER);

  g_pcsAuto->AsignarID(AUTO);
  // El auto es esttico hasta que el ladrn
  // se logra subir
  g_pcsAuto->AsignarcBmpAlternativo(g_pAutoRobado);

  // se guarda la posicin del auto en la matriz
  // de destinos
  g_rDestinos[g_iDestinos] = g_pcsAuto->ObtenerPosicion();

  g_pJuego->AgregarSprite(g_pcsAuto);

  // Se coloca la mirilla
  // Objeto mirilla
  g_pcsMirilla = new cSprite(g_pMirilla, rcAreaJuego, AFL_DETENER);
  g_pcsMirilla->AsignarID(MIRILLA);
  // la mirilla est por sobre el resto de los sprites
  g_pcsMirilla->AsignarOrdenZ(6);
  // no entra en el control de colisiones
  g_pcsMirilla->AsignarDensidad(0);
  // se grega el sprite
  g_pJuego->AgregarSprite(g_pcsMirilla);

  // Coloca los tomates
  // que actan como contadores de
  // los tomates utilizados para detener a los ladrones
  for (int i = 0; i < g_iQShots ; i++)
  {
    g_pcsTomate[i] = new cSprite(g_pTomate, rcAreaJuego, AFL_DESAPARECER);
    g_pcsTomate[i]->AsignarOrdenZ(5);
    g_pcsTomate[i] ->AsignarID(TOMATE);
    g_pcsTomate[i]->AsignarPosicion(i * 12, 460);
    g_pcsTomate[i]->AsignarcBmpAlternativo(g_pImpactoTomate);
    g_pJuego->AgregarSprite(g_pcsTomate[i]);
   }

  // Contamos la cantidad de ciclos
  g_iContBucle = 0;
  // Reinicia la reproduccin de la msica de fondo
  g_pJuego->MIDI(PLAYMIDI, TEXT("fondo.mid"));
}
