//-----------------------------------------------------------------
// Clase cFondo
// Cdigo fuente - cFondo.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "cFondo.h"
#include "cGestorFondos.h"

//-----------------------------------------------------------------
// cFondo Constructores y Destructor
//-----------------------------------------------------------------
// Crea un mapa de bits a partir de un archivo bmp
cFondo::cFondo(HDC hDC, LPTSTR szFileName)
    : cBMP(hDC, szFileName)
{
  // Asume que la ventana visible
  // coincide con el tamao del mapa de bits
  // despus se puede ajustar
  m_rcVisible.left = m_rcVisible.top = m_ptCoordViewport.x = m_ptCoordViewport.y = 0;
  m_rcVisible.right = m_eInfoHeader.biWidth;
  m_rcVisible.bottom = m_eInfoHeader.biHeight;
  // se asume el punto 0,0 respecto al rea del juego
  m_ptVelocRelativa.x = m_ptVelocRelativa.y = 0;
}

// Crea un fondo a partir de un recurso bmp
cFondo::cFondo(HDC hDC, UINT uiIDRecurso, HINSTANCE hInstancia)
   : cBMP(hDC, uiIDRecurso, hInstancia)
{
  // Asume que la ventana visible
  // coincide con el tamao del mapa de bits
  // despus se puede ajustar
  m_rcVisible.left = m_rcVisible.top = m_ptCoordViewport.x = m_ptCoordViewport.y = 0;
  m_rcVisible.right = m_eInfoHeader.biWidth;
  m_rcVisible.bottom = m_eInfoHeader.biHeight;
  // se asume el punto 0,0 respecto al rea del juego
  m_ptVelocRelativa.x = m_ptVelocRelativa.y = 0;
}

// Destructor
cFondo::~cFondo()
{
  Liberar();
}

//-----------------------------------------------------------------
// Mtodos auxiliares
//-----------------------------------------------------------------

POINT cFondo::Actualizar(POINT ptVeloObjetoRef, WORD  wLimite)
{

  // velocidad x positiva-> mueve el rectngulo
  // visible hacia la derecha
  // velocidad x negativa-> mueve el rectngulo
  // visible hacia la izquierda
  m_ptCoordViewport.x += (m_ptVelocRelativa.x + ptVeloObjetoRef.x);
  m_ptCoordViewport.y += (m_ptVelocRelativa.y + ptVeloObjetoRef.y);

  // Guarda las coordenadas despus de modificarlas
  // para despus verificar si finalmente la velocidad
  // se aplica en su totalidad o no
  POINT ptRef = m_ptCoordViewport;

  m_rcVisible.left += (m_ptVelocRelativa.x + ptVeloObjetoRef.x);
  m_rcVisible.right += (m_ptVelocRelativa.x + ptVeloObjetoRef.x);
  // velocidad y positiva-> mueve el rectngulo
  // visible hacia abajo
  // velocidad y negativa-> mueve el rectngulo
  // visible hacia arriba
  m_rcVisible.top += (m_ptVelocRelativa.y + ptVeloObjetoRef.y);
  m_rcVisible.bottom += (m_ptVelocRelativa.y + ptVeloObjetoRef.y);

  SIZE sizTotal;
  // ancho y alto del rectngulo visible
  sizTotal.cx  = m_rcVisible.right - m_rcVisible.left;
  sizTotal.cy = m_rcVisible.bottom - m_rcVisible.top;

  if (wLimite == SIN_TOPE)
  {
    // si se llega al lmite lateral derecho del mapa de bits
    if (m_rcVisible.left > m_eInfoHeader.biWidth)
    {
        m_rcVisible.left = 0;
        m_rcVisible.right = sizTotal.cx;
    }
    // si se llega al lmite lateral izquierdo del mapa de bits
    if (m_rcVisible.right < 0)
    {
        m_rcVisible.left = m_eInfoHeader.biWidth - sizTotal.cx;
        m_rcVisible.right = m_eInfoHeader.biWidth;
    }
    // si se llega al lmite inferior del mapa de bits
    if (m_rcVisible.top > m_eInfoHeader.biHeight)
    {
        m_rcVisible.top = 0;
        m_rcVisible.bottom = sizTotal.cy;
    }
    // si se llega al lmite superior del mapa de bits
    if (m_rcVisible.bottom < 0)
    {
        m_rcVisible.top = m_eInfoHeader.biHeight - sizTotal.cy;
        m_rcVisible.bottom =  m_eInfoHeader.biHeight;
    }
  }
  else
  {
    if (m_rcVisible.right > m_eInfoHeader.biWidth &&
            wLimite & TOPE_ESTE)
    // si se llega al lmite lateral derecho del mapa de bits
    {
        // se ajustan los valores del posicionamiento
        // dado que hay una correccin por haber
        // llegado a un lmite
        // tambin se detiene la velocidad en el eje x
        m_ptCoordViewport.x -= (m_rcVisible.right - m_eInfoHeader.biWidth);
        m_rcVisible.right = m_eInfoHeader.biWidth;
        m_rcVisible.left = m_eInfoHeader.biWidth - sizTotal.cx;
    }
    // si se llega al lmite lateral izquierdo del mapa de bits
    if (m_rcVisible.left < 0  &&  wLimite & TOPE_OESTE)
    {
        // se ajustan los valores del posicionamiento
        // dado que hay una correccin por haber
        // llegado a un lmite
        // tambin se detiene la velocidad en el eje x
        m_ptCoordViewport.x -= m_rcVisible.left;
        m_rcVisible.right = sizTotal.cx;
        m_rcVisible.left = 0;
    }
    // si se llega al lmite inferior del mapa de bits
    if (m_rcVisible.bottom > m_eInfoHeader.biHeight &&
            wLimite & TOPE_SUR)
    {
        // se ajustan los valores del posicionamiento
        // dado que hay una correccin por haber
        // llegado a un lmite
        // tambin se detiene la velocidad en el eje y
        m_ptCoordViewport.y -= (m_rcVisible.bottom - m_eInfoHeader.biHeight);
        m_rcVisible.top = m_eInfoHeader.biHeight - sizTotal.cy;
        m_rcVisible.bottom = m_eInfoHeader.biHeight;
    }
    // si se llega al lmite superior del mapa de bits
    if (m_rcVisible.top < 0 && wLimite & TOPE_NORTE)
    {
        // se ajustan los valores del posicionamiento
        // dado que hay una correccin por haber
        // llegado a un lmite
        // tambin se detiene la velocidad en el eje y
        m_ptCoordViewport.y -= m_rcVisible.top;
        m_rcVisible.top = 0;
        m_rcVisible.bottom =  sizTotal.cy;
    }
  }
  // Verifica si hubo correccin de la
  // velocidad aplicada por haber llegado
  // a algn lmite
  POINT ptVeloAplicada = ptVeloObjetoRef;

  if (m_ptCoordViewport.x != ptRef.x)
      ptVeloAplicada.x = ptVeloObjetoRef.x - m_ptCoordViewport.x - ptRef.x;
  if (m_ptCoordViewport.y != ptRef.y)
      ptVeloAplicada.y = ptVeloObjetoRef.y - m_ptCoordViewport.y - ptRef.y;

  // Devuelve la velocidad realmente aplicada
  // es un dato importante cuando se trata del
  // fondo principal
  return ptVeloAplicada;

}


int cFondo::Dibujar(HDC hDC)
{

  POINT ptVentana;
  POINT ptBmpOrigen;
  SIZE sizTotal;
  sizTotal.cx  = m_rcVisible.right - m_rcVisible.left;
  sizTotal.cy = m_rcVisible.bottom - m_rcVisible.top;

  SIZE sizDibujo;
  // X e Y son el ancho y la altura del mapa de bits
  // de fondo
  int X = m_eInfoHeader.biWidth;
  int Y = m_eInfoHeader.biHeight;

  // CASO 1
  // Rectngulo sobresale por esquina superior izquierda
  // del mapa de bits del fondo
  if (m_rcVisible.left < 0 && m_rcVisible.top < 0 )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 1 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = X - abs(m_rcVisible.left);
      ptBmpOrigen.y = Y - abs(m_rcVisible.top);
      sizDibujo.cx = abs(m_rcVisible.left);
      sizDibujo.cy = abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 1 sub rectngulo 2
      ptVentana.x = abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.right);
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 1 sub rectngulo 3
      // ptVentana.x mantiene el valor anterior
      ptVentana.y = abs(m_rcVisible.top);
      // ptBmpOrigen.x mantiene el valor anterior
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor anterior
      sizDibujo.cy = abs(m_rcVisible.bottom);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 1 sub rectngulo 4
      ptVentana.x = 0;
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = X - abs(m_rcVisible.left);
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.left);
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
      return 1;
  }
  // CASO 2
  // Rectngulo sobresale por esquina superior derecha
  // del mapa de bits del fondo
  else if (m_rcVisible.right > X  && m_rcVisible.top < 0 )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 2 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = Y - abs(m_rcVisible.top);
      sizDibujo.cx = X - abs(m_rcVisible.left);
      sizDibujo.cy = abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 2 sub rectngulo 2
      ptVentana.x = X - abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.right) - X;
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 2 sub rectngulo 3
      // ptVentana.x mantiene el valor anterior
      ptVentana.y = abs(m_rcVisible.top);
      // ptBmpOrigen.x mantiene el valor anterior
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor anterior
      sizDibujo.cy = abs(m_rcVisible.bottom);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 2 sub rectngulo 4
      ptVentana.x = 0;
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = 0;
      sizDibujo.cx = X - abs(m_rcVisible.left);
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
        return 2;
  }
  // CASO 3
  // Rectngulo sobresale por esquina inferior derecha
  // del mapa de bits
  else if (m_rcVisible.right > X  && m_rcVisible.bottom > Y )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 3 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = X - abs(m_rcVisible.left);
      sizDibujo.cy = Y - abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 3 sub rectngulo 2
      ptVentana.x = X - abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor
      sizDibujo.cx = abs(m_rcVisible.right) - X;
      // sizDibujo.cy mantiene el valor
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 3 sub rectngulo 3
      // ptVentana.x mantiene el valor anterior
      ptVentana.y = Y - abs(m_rcVisible.top);
      // ptBmpOrigen.x mantiene el valor
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor
      sizDibujo.cy = abs(m_rcVisible.bottom) -Y;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 3 sub rectngulo 4
      ptVentana.x = 0;
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = abs(m_rcVisible.left);
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = X - abs(m_rcVisible.left);
      // sizDibujo.cy  mantiene el valor
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 3;
  }
  // CASO 4
  // Rectngulo sobresale por esquina inferior izquierda
  // del mapa de bits
  else if (m_rcVisible.left < 0  && m_rcVisible.bottom > Y )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 4 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = X - abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = abs(m_rcVisible.left);
      sizDibujo.cy = Y - abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 4 sub rectngulo 2
      ptVentana.x = abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor
      sizDibujo.cx = abs(m_rcVisible.right);
      // sizDibujo.cy mantiene el valor
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 4 sub rectngulo 3
      // ptVentana.x mantiene el valor anterior
      ptVentana.y = Y - abs(m_rcVisible.top);
      // ptBmpOrigen.x mantiene el valor
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor
      sizDibujo.cy = abs(m_rcVisible.bottom) - Y;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 4 sub rectngulo 4
      ptVentana.x = 0;
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = X - abs(m_rcVisible.left);
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.left);
      // sizDibujo.cy  mantiene el valor
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 4;
  }
  // CASO 5
  // Rectngulo sobresale por arriba
  else if (m_rcVisible.top < 0 )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 5 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = Y - abs(m_rcVisible.top);
      sizDibujo.cx = sizTotal.cx;
      sizDibujo.cy = abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 5 sub rectngulo 2
      // ptVentana.x mantiene el valor
      ptVentana.y = abs(m_rcVisible.top);
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor anterior
      sizDibujo.cy =  abs(m_rcVisible.bottom);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 5;
  }
  // CASO 6
  // Rectngulo sobresale por lateral derecho
  else if (m_rcVisible.right > X )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 6 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = X - abs(m_rcVisible.left);
      sizDibujo.cy = sizTotal.cy;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 6 sub rectngulo 2
      ptVentana.x = X - abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.right) - X;
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 6;
  }
  // CASO 7
  // Rectngulo sobresale por abajo
  else if (m_rcVisible.bottom > Y )
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 7 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = sizTotal.cx;
      sizDibujo.cy = Y - abs(m_rcVisible.top);
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));


      // Rectngulo 7 sub rectngulo 2
      //ptVentana.x mantiene el valor anterior
      ptVentana.y = Y - abs(m_rcVisible.top);
      // ptBmpOrigen.x mantiene el valor anterior
      ptBmpOrigen.y = 0;
      // sizDibujo.cx mantiene el valor anterior
      sizDibujo.cy = abs(m_rcVisible.bottom) -Y;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 7;
  }
  // CASO 8
  // Rectngulo sobresale por lateral izquierdo
  // del mapa de bits
  else if (m_rcVisible.left < 0)
  {
      // Los dibujos se hacen todos en la funcin DibujoParcial
      // Rectngulo 8 sub rectngulo 1
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = X - abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = abs(m_rcVisible.left);
      sizDibujo.cy = sizTotal.cy;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));

      // Rectngulo 8 sub rectngulo 2
      ptVentana.x = abs(m_rcVisible.left);
      // ptVentana.y mantiene el valor anterior
      ptBmpOrigen.x = 0;
      // ptBmpOrigen.y mantiene el valor anterior
      sizDibujo.cx = abs(m_rcVisible.right);
      // sizDibujo.cy mantiene el valor anterior
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 8;
  }
  // CASO simple
  // Rectngulo visible en el medio del mapa de bits
  else
  {
      // Un solo dibujo
      ptVentana.x = 0;
      ptVentana.y = 0;
      ptBmpOrigen.x = abs(m_rcVisible.left);
      ptBmpOrigen.y = abs(m_rcVisible.top);
      sizDibujo.cx = sizTotal.cx;
      sizDibujo.cy = sizTotal.cy;
      DibujoParcial(hDC, ptVentana, ptBmpOrigen , sizDibujo,
            true, RGB(255,0,255));
            return 9;
  }
 }

void cFondo::TransportarViewport(POINT ptTransporte)
{

  m_ptCoordViewport.x +=  ptTransporte.x;
  m_ptCoordViewport.y +=  ptTransporte.y;

  m_rcVisible.left +=  ptTransporte.x;
  m_rcVisible.right += ptTransporte.x;
  m_rcVisible.top += ptTransporte.y;
  m_rcVisible.bottom += ptTransporte.y;

}








