//-----------------------------------------------------------------
// Clase cGestorFondos
// Cdigo fuente - cGestorFondos.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "cGestorFondos.h"

//-----------------------------------------------------------------
// cGestorFondos Constructores y Destructor
//-----------------------------------------------------------------
cGestorFondos::cGestorFondos(int iAncho, int iAltura, WORD lfLimite)
{
  // Asignacin del tamao del viewport
  SetRect(&m_rcViewportSize,0 ,0 ,iAncho, iAltura);

  // Asignacin de cantidad de fondos
  m_iQFondos = 0;

  // Opcin al llegar al lmite del mapa de bits
  // de cada fondo
  m_lfLimite = lfLimite;

  // Referencia al punto 0, 0 respecto al rea de juego
  m_ptCoordViewport.x = m_ptCoordViewport.y = 0;

  // Fondo master
  // presupone que es el 0
  m_iFondoMaster = 0;
}

cGestorFondos::~cGestorFondos()
{
}

//-----------------------------------------------------------------
// Mtodos generales
//-----------------------------------------------------------------
bool cGestorFondos::AgregarFondo(cFondo* pFondo, bool bMaster)
{

    // se controla que no se asignen mas fondos
    // de los que se pueden gestionar
    if (m_iQFondos > LIMITE_FONDOS)
        return false;

    // se asigna el puntero a la matriz de punteros
    m_pFondos[m_iQFondos] = pFondo;

    // Informa el rectngulo que se visualiza
    pFondo->AsignarRcVisible(m_rcViewportSize);

    // se indica que este fondo es el principal
    // (el que sigue el movimiento de algn objeto)
    if (bMaster)
        m_iFondoMaster = m_iQFondos;

    // se incrementa el contador de fondos creados
    m_iQFondos += 1;

    // para informar que el fondo se agreg
    // correctamente
    return true;
}

// Actualizar()
void cGestorFondos::Actualizar(POINT ptVeloObjetoRef)
{
    // actualiza la posicin de los fondos
    // tomando en cuenta un objeto de referencia
    POINT ptVeloAplicada;

    // Actualiza el fondo principal
    // se actualiza la posicin del fondo
    // teniendo en cuenta el tipo de lmite
    // la velocidad del objeto de referencia se suma a
    // la velocidad relativa del fondo
    ptVeloAplicada = m_pFondos[m_iFondoMaster]->Actualizar(ptVeloObjetoRef, m_lfLimite);

    // Obtiene la referencia absoluta actualizada
    // respecto al punto 0 0
    m_ptCoordViewport = m_pFondos[m_iFondoMaster]->ObtenerCoordViewport();

    // se actualiza la posicin de los otros
    // fondos teniendo en cuenta el tipo de lmite
    // y pasando en cada caso la velocidad del objeto
    // de referencia
    for (int i = 0; i < m_iQFondos; i++)
    {
        if (i != m_iFondoMaster)
            m_pFondos[i]->Actualizar(ptVeloAplicada, m_lfLimite);
    }

}
bool cGestorFondos::Dibujar(HDC hDC, int iFondoIni, int iFondoFin)
{
    // se dibujan los fondos que correspondan
    // de esta manera se pueden dibujar fondos antes y
    // despus de los sprites del juego
    // si no hay fondos, se retorna sin hacer nada
    if (m_iQFondos == 0)
        return true;

    // se comprueban los parmetros para evitar
    // errores
    if (iFondoIni > m_iQFondos ||
        iFondoIni < 1          ||
        iFondoFin > m_iQFondos ||
        iFondoFin < iFondoIni)
        return false;

    // se dibujan los fondos solicitados
    for (int i = iFondoIni - 1; i < iFondoFin ; i++)
        m_pFondos[i]->Dibujar(hDC);

    // se puede controlar que el valor
    // de retorno sea true
    return true;
}

void cGestorFondos::TransportarViewport(POINT ptCoord)
{
    // Asigna el rectngulo visible para los
    // distintos fondos
    for (int i = 0; i < m_iQFondos; i++)
        m_pFondos[i]->TransportarViewport(ptCoord);

}
void cGestorFondos::ReposicionarViewport()
{
    // reposiciona los fondos
    // al punto de inicio
    for (int i = 0; i < m_iQFondos; i++)
        m_pFondos[i]->AsignarRcVisible(m_rcViewportSize);

}
RECT cGestorFondos::ObtenerViewport()
{
    // Calcula el viewport vigente
    RECT rc = m_rcViewportSize;
    rc.left += m_ptCoordViewport.x;
    rc.right += m_ptCoordViewport.x;
    rc.top += m_ptCoordViewport.y;
    rc.bottom += m_ptCoordViewport.y;

    return rc;

}
void cGestorFondos::EliminarFondos()
{
    // Elimina todos los fondos
    // para permitir una
    // redefinicin
    m_iQFondos = 0;       // Cantidad de fondos
    m_ptCoordViewport.x = m_ptCoordViewport.y = 0;
    m_iFondoMaster = 0;   // fondo principal

}

