//-----------------------------------------------------------------
// Objeto cFondo
// Archivo de encabezado - cFondo.h
//-----------------------------------------------------------------

#pragma once

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include <windows.h>
#include "cBMP.h"
#include <math.h>

//-----------------------------------------------------------------
// Clase cFondo
//-----------------------------------------------------------------
class cFondo : public cBMP
{
protected:
  // Variables miembro protegidos

  // Rectngulo de vista del mapa de bits
  // siempre es un rectngulo referido al
  // mapa de bits
  RECT          m_rcVisible;

  // Velocidad en eje x y velocidad en eje y
  // expresado con un objeto POINT
  // es la velocidad relativa entre los distintos fondos
  // si fuese 0 ,0 el fondo de mover simplemente a la
  // velocidad del objeto de referencia, en caso contrario,
  // esta velocidad se suma a la velocidad  del objeto
  // de referencia
  POINT         m_ptVelocRelativa;

  // Punto de referencia absoluta al punto 0,0
  // coordenadas del juego
  POINT         m_ptCoordViewport;

private:
 // Asignacin y obtencin del rectngulo del viewport
 void    AsignarRcVisible(RECT& rcVisible)
            { CopyRect(&m_rcVisible, &rcVisible);
              m_ptCoordViewport.x = rcVisible.left;
              m_ptCoordViewport.y = rcVisible.top; };

 // Transporta el viewport una cantidad determinada
 // de pxeles
void     TransportarViewport(POINT ptCoord);

// La clase cGestorFondos es la que puede utilizar
// las funciones private
friend  class  cGestorFondos;

public:
  // Constructores y Destructor

  // Este constructor asume
  // posicin inicial = centrado en el mapa de bits

  // Constructor a partir de un archivo bmp
  cFondo(HDC hDC, LPTSTR szFileName);

  // Constructor a partir de un recurso de la aplicacin
  cFondo(HDC hDC, UINT uiIDRecurso, HINSTANCE hInstance);

  // Destructor
  ~cFondo();

  // Mtodos

  // Dibujo del bitmap
  virtual int Dibujar(HDC hDC);

  // Actualiza la parte visible del mapa de bits
  virtual POINT Actualizar(POINT ptVeloObjetoRef, WORD lfLimite);

  // Asignacin y obtencin de la velocidad
  // Dos modos de asignar velocidad: x-y  o POINT
  void    AsignarVelocRelativa(int x, int y)
        {m_ptVelocRelativa.x = x; m_ptVelocRelativa.y = y; };

  void    AsignarVelocRelativa(POINT ptVelocRelativa)
        {m_ptVelocRelativa = ptVelocRelativa;};

  POINT   ObtenerVelocRelativa()      { return m_ptVelocRelativa; };

  // Informa la referencia absoluta
  POINT   ObtenerCoordViewport()        { return m_ptCoordViewport; };

  // Informa el rectngulo visible del fondo
  RECT&   ObtenerRcVisible()          { return m_rcVisible; };

};
